#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

static int
exec_wait (const char *path, const char *av[])
{
	pid_t   pid = vfork ();

	if (pid < 0)
		return 1;
	if (!pid)
	{
		execv (path, (char *const *) av);
		_exit (1);
	} else
	{
		int     status = 0;

		if (waitpid (pid, &status, 0) != pid || !WIFEXITED (status))
			return 1;
		return WEXITSTATUS (status);
	}
}

int
main (int ac, const char *av[])
{
	int     rc;
	const char *fixpost_args[] = { "glibc_fix_post", 0 };
	const char *ldconfig_args[] = { "ldconfig", 0 };
	const char *telinit_args[] = { "telinit", "u", 0 };
	const char *telinit_path = "/sbin/telinit";
	const char *p;
	char    initpath[4096];

	if ((p = getenv ("RPM_INSTALL_ARG1")) && (*p != '0') && (*p != '1'))
		(void) exec_wait ("/sbin/glibc_fix_post", fixpost_args);

	if ((rc = exec_wait ("/sbin/post_ldconfig", ldconfig_args)))
		return rc;

	/* Check if telinit is available and the init fifo as well.  */
	if (access (telinit_path, X_OK) || access ("/dev/initctl", F_OK))
		return 0;

	/* Check if we are not inside of some chroot.  */
	if (readlink ("/proc/1/exe", initpath, sizeof initpath) <= 0 ||
	    readlink ("/proc/1/root", initpath, sizeof initpath) <= 0)
		return 0;

	return exec_wait (telinit_path, telinit_args);
}
